#!/usr/bin/env python
################################################################################
#
# Script: cdb_tar.py
#
# Purpose: Script to create a tar pkg for a CDBS delivery.
#
#       The release catalog and reference files are put into
#       a gzipped tar file.  Hard-links are created from the
#       outgoing copy directories for each remote site to the
#       version created here.
#
# Modification History:
#
# DATE     PR     AUTHOR    REASON
# -------- -----  --------  --------------------------------------
# 04/11/03 48141  MSwam     first version
# 05/06/04 48141  MSwam     add timestamp to trailer start message
# 05/20/10 44712  MSwam     add ls -l to see file size after gzip
# 03/09/11 64748  RBelt     Rewrote code in python.  Old program was
#                           written in shell script.  Tar files that produce
#                           a file set > 4GB were not being accessible in 
#                           32 bit systems.  Any file sets that are greater
#                           than 4 GB will be broken into multiple tar files
# 03/16/11 64748  RBelt     Fixed bug with WFPC-2 deliveries coming in sets 
# 05/26/11 64748  MSwam     Make trailer file open in 'append' mode
# 10/19/11 64748  MSwam     Shrink tar-ball limit down to 1.2Gb to hopefully
#                             head-off ftp problems to the remote sites
#
################################################################################

import os,sys,re,time,commands
import tarfile
from PipelineUtils import evaluateDefinitionsFile
from datetime import datetime

max_size           = 1200000000  # Lets use 1.2Gb, instead of 4294967296
go_absent          = 102
quit_this_database = 1
file_groups        = []
input_dir          = os.environ['CDB_IN_DIR']
osf_dataset        = os.environ['OSF_DATASET']
cat_file_ext       = os.environ['CATALOG_FILE_EXT']
output1_dir        = os.environ['OUTGOING1_DIR']
output2_dir        = os.environ['OUTGOING2_DIR']

"""
Method: get_size()
Parameter: file
Purpose: To get size of file
Returns: Size of file
"""
def get_size(file):
  return os.path.getsize(file)
  
"""
Method: get_group_size()
Parameter: group --> Array of files
Purpose: To get the total file size of a group of files
Returns: Integer that represents collective size of files in bytes
"""
def get_group_size(group):
  size = 0
  for file in group:
    size += get_size(file)
  return size

"""
Method: break_up_files()
Parameter: file_list --> Array of files
Purpose: Will split each array in which the collective size of files
         within the array is greater than max_size.  This will create a 
         new array of file lists in which the collective size of each
         list will be less than max_size.
Returns: Single array in which each element is an array of files with a 
         collective size of less than max_size.
""" 

def break_up_files(file_list):
  file_group1 = []
  file_group2 = []

  file_group1 = file_list[:(len(file_list)/2)]
  file_group2 = file_list[(len(file_list)/2):]

  if(get_group_size(file_group1) > max_size):
    break_up_files(file_group1)
  else:
    file_groups.append(file_group1)

  if(get_group_size(file_group2) > max_size):
    break_up_files(file_group2)
  else:
    file_groups.append(file_group2)

# Print Banner and current date/time
print ">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>"
print datetime.today()

# Check if input directory exists and change current working directory
if(0 == os.path.exists(input_dir)):
  print "ERROR: " + input_dir + " directory does not exist"
  sys.exit(go_absent)
try: 
  os.chdir(input_dir)
except:
  print "ERROR: Failed to set directory to ",input_dir
print "Sitting in " + os.getcwd()

#Open trailer file for appending and print heading for cdbtar process
trl_file = open(os.environ['OSF_DATASET'] + ".trl",'a')
now = datetime.today()
trl_file.write("CDBTAR===" + now.strftime('%d-%b-%Y %H:%M:%S') + "===creating tar file\n")

# Open OSF_DATASET file for extraction of fits filenames / extract filenames
infile_handle = ''
try:
  infile_handle = open(osf_dataset + cat_file_ext)
except:
  print "ERROR: Unable to open OSF_DATASET file: " + osf_dataset + cat_file_ext
  sys.exit(go_absent)
list_of_files = []
dir_list = os.listdir(os.getcwd())
for line in infile_handle:
  if " I " in line:
    rawname = line[:line.find(' ')].lower()
    for file in os.listdir(os.getcwd()):
      if rawname in file and file not in list_of_files:
        list_of_files.append(file)
list_of_files.append(osf_dataset + cat_file_ext)

total_size = 0
new_list = []

# Loop through file list and determine total size to check of > 4GB
# If file group is too large will recursively create multiple deliveries
for file in list_of_files:
  total_size += os.path.getsize(file)
if(total_size > max_size):
  break_up_files(list_of_files)
else:
  file_groups.append(list_of_files)
print "Total number of groups required: ",len(file_groups)

# Create tar files for all groups (Remove old ones)
grp_ctr = 1
for group in file_groups:
  tar_file = ''
  try:
    tar_file = osf_dataset + ".tar.gz_" + str(grp_ctr)
    if(os.path.exists(tar_file)):
      os.remove(tar_file)
    tar_handle = tarfile.open(tar_file,'w:gz')
    for file in group:
      tar_handle.add(file,arcname=file)
      trl_file.write(commands.getstatus(tar_file))
    tar_handle.close()
    print commands.getstatus(tar_file)

# Create hard links for each new tar/gzipped file (Remove old ones)
    if(os.path.exists(output1_dir + tar_file)):
      os.remove(output1_dir + tar_file)
    os.link(tar_file,output1_dir + tar_file)
    if(os.path.exists(output2_dir + tar_file)):
      os.remove(output2_dir + tar_file)
    os.link(tar_file,output2_dir + tar_file)
    grp_ctr+=1
  except:
     trl_file.write("ERROR: Failed to create tar file for " + tar_file)
     trl_file.close()
     sys.exit(quit_this_dataset)
trl_file.close()
